@@ -15,8 +15,14 @@ to the system log.
 */
 
 /*
- * $Id: log-server.c,v 1.3 1995/08/21 23:25:00 ylo Exp $
+ * $Id: log-server.c,v 1.5 1995/10/02 01:22:57 ylo Exp $
  * $Log: log-server.c,v $
+ * Revision 1.5  1995/10/02  01:22:57  ylo
+ * 	Include sys/syslog.h if needed.
+ *
+ * Revision 1.4  1995/09/09  21:26:42  ylo
+ * /m/shadows/u2/users/ylo/ssh/README
+ *
  * Revision 1.3  1995/08/21  23:25:00  ylo
  * 	Added support for syslog facility.
  *
@@ -29,7 +35,11 @@ to the system log.
 
 #include "includes.h"
 #include <syslog.h>
+#ifdef NEED_SYS_SYSLOG_H
+#include <sys/syslog.h>
+#endif /* NEED_SYS_SYSLOG_H */
 #include "packet.h"
+#include "xmalloc.h"
 #include "ssh.h"
 
 static int log_debug = 0;
@@ -108,8 +118,8 @@ void log(const char *fmt, ...)
   vsprintf(buf, fmt, args);
   va_end(args);
   if (log_on_stderr)
-    fprintf(stderr, "%s\n", buf);
-  syslog(LOG_INFO, "%s", buf);
+    fprintf(stderr, "log: %s\n", buf);
+  syslog(LOG_INFO, "log: %.500s", buf);
 }
 
 /* Debugging messages that should not be logged during normal operation. */
@@ -124,8 +134,8 @@ void debug(const char *fmt, ...)
   vsprintf(buf, fmt, args);
   va_end(args);
   if (log_on_stderr)
-    fprintf(stderr, "%s\n", buf);
-  syslog(LOG_DEBUG, "%s", buf);
+    fprintf(stderr, "debug: %s\n", buf);
+  syslog(LOG_DEBUG, "debug: %.500s", buf);
 }
 
 /* Error messages that should be logged. */
@@ -140,8 +150,50 @@ void error(const char *fmt, ...)
   vsprintf(buf, fmt, args);
   va_end(args);
   if (log_on_stderr)
-    fprintf(stderr, "%s\n", buf);
-  syslog(LOG_ERR, "%s", buf);
+    fprintf(stderr, "error: %s\n", buf);
+  syslog(LOG_ERR, "error: %.500s", buf);
+}
+
+struct fatal_cleanup
+{
+  struct fatal_cleanup *next;
+  void (*proc)(void *);
+  void *context;
+};
+
+static struct fatal_cleanup *fatal_cleanups = NULL;
+
+/* Registers a cleanup function to be called by fatal() before exiting. */
+
+void fatal_add_cleanup(void (*proc)(void *), void *context)
+{
+  struct fatal_cleanup *cu;
+
+  cu = xmalloc(sizeof(*cu));
+  cu->proc = proc;
+  cu->context = context;
+  cu->next = fatal_cleanups;
+  fatal_cleanups = cu;
+}
+
+/* Removes a cleanup frunction to be called at fatal(). */
+
+void fatal_remove_cleanup(void (*proc)(void *context), void *context)
+{
+  struct fatal_cleanup **cup, *cu;
+  
+  for (cup = &fatal_cleanups; *cup; cup = &cu->next)
+    {
+      cu = *cup;
+      if (cu->proc == proc && cu->context == context)
+	{
+	  *cup = cu->next;
+	  xfree(cu);
+	  return;
+	}
+    }
+  fatal("fatal_remove_cleanup: no such cleanup function: 0x%lx 0x%lx\n",
+	(unsigned long)proc, (unsigned long)context);
 }
 
 /* Fatal messages.  This function never returns. */
@@ -150,20 +202,30 @@ void fatal(const char *fmt, ...)
 {
   char buf[1024];
   va_list args;
+  struct fatal_cleanup *cu, *next_cu;
+  static int fatal_called = 0;
+
   if (log_quiet)
     exit(1);
   va_start(args, fmt);
   vsprintf(buf, fmt, args);
   va_end(args);
   if (log_on_stderr)
-    fprintf(stderr, "%s\n", buf);
-  syslog(LOG_NOTICE, "%s", buf);
+    fprintf(stderr, "fatal: %s\n", buf);
+  syslog(LOG_ERR, "fatal: %.500s", buf);
 
-  /* Unlink any X11 sockets if necessary. */
-  channel_stop_listening();
+  if (fatal_called)
+    exit(1);
+  fatal_called = 1;
 
-  /* Close the connection to the client. */
-  packet_close();
+  /* Call cleanup functions. */
+  for (cu = fatal_cleanups; cu; cu = next_cu)
+    {
+      next_cu = cu->next;
+      debug("Calling cleanup 0x%lx(0x%lx)",
+	    (unsigned long)cu->proc, (unsigned long)cu->context);
+      (*cu->proc)(cu->context);
+    }
 
   exit(1);
 }
