@@ -8,10 +8,25 @@ Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
                    All rights reserved
 
 Created: Fri Mar 17 05:03:28 1995 ylo
-Last modified: Wed Jun 21 22:12:21 1995 ylo
+
+Functions for allocating a pseudo-terminal and making it the controlling
+tty.
 
 */
 
+/*
+ * $Id: pty.h,v 1.3 1995/07/16 01:03:40 ylo Exp $
+ * $Log: pty.h,v $
+ * Revision 1.3  1995/07/16  01:03:40  ylo
+ * 	Added pty_release.
+ *
+ * Revision 1.2  1995/07/13  01:28:41  ylo
+ * 	Removed "Last modified" header.
+ * 	Added cvs log.
+ *
+ * $Endlog$
+ */
+
 #ifndef PTY_H
 #define PTY_H
 
@@ -21,6 +36,10 @@ Last modified: Wed Jun 21 22:12:21 1995 ylo
    returned (the buffer must be able to hold at least 64 characters). */
 int pty_allocate(int *ptyfd, int *ttyfd, char *ttyname);
 
+/* Releases the tty.  Its ownership is returned to root, and permissions to
+   0666. */
+void pty_release(const char *ttyname);
+
 /* Makes the tty the processes controlling tty and sets it to sane modes. 
    This may need to reopen the tty to get rid of possible eavesdroppers. */
 void pty_make_controlling_tty(int *ttyfd, const char *ttyname);
