@@ -14,8 +14,11 @@ Functions for reading the configuration file.
 */
 
 /*
- * $Id: readconf.h,v 1.3 1995/07/27 00:39:10 ylo Exp $
+ * $Id: readconf.h,v 1.4 1995/09/24 23:59:57 ylo Exp $
  * $Log: readconf.h,v $
+ * Revision 1.4  1995/09/24  23:59:57  ylo
+ * 	Added connection_attempts.
+ *
  * Revision 1.3  1995/07/27  00:39:10  ylo
  * 	Added GlobalKnownHostsFile and UserKnownHostsFile.
  *
@@ -50,10 +53,17 @@ typedef struct
   int password_authentication;	/* Try password authentication. */
   int fallback_to_rsh;		/* Use rsh if cannot connect with ssh. */
   int use_rsh;			/* Always use rsh (don\'t try ssh). */
+  int batch_mode;		/* Batch mode: do not ask for passwords. */
+  int strict_host_key_checking;	/* Strict host key checking. */
+  int compression;		/* Compress packets in both directions. */
+  int compression_level;	/* Compression level 1 (fast) to 9 (best). */
+  int keepalives;		/* Set SO_KEEPALIVE. */
 
   int port;			/* Port to connect. */
+  int connection_attempts;	/* Max attempts (seconds) before giving up */
   int cipher;			/* Cipher to use. */
   char *hostname;		/* Real host to connect. */
+  char *proxy_command;		/* Proxy command for connecting the host. */
   char *user;			/* User to log in as. */
   int escape_char;		/* Escape character; -2 = none */
 
