@@ -13,6 +13,12 @@ Definitions for server configuration data and for the functions reading it.
 
 */
 
+/*
+ * $Id$
+ * $Log$
+ * $EndLog$
+ */
+
 #ifndef SERVCONF_H
 #define SERVCONF_H
 
@@ -24,18 +30,24 @@ typedef struct
   int port;			/* Port number to listen on. */
   struct in_addr listen_addr;	/* Address on which the server listens. */
   char *host_key_file;		/* File containing host key. */
+  char *random_seed_file;	/* File containing random seed. */
   int server_key_bits;		/* Size of the server key. */
   int login_grace_time;		/* Disconnect if no auth in this time (sec). */
   int key_regeneration_time;	/* Server key lifetime (seconds). */
   int permit_root_login;	/* If true, permit root login. */
+  int ignore_rhosts;		/* Ignore .rhosts and .shosts. */
   int quiet_mode;		/* If true, don't log anything but fatals. */
   int fascist_logging;		/* Perform very verbose logging. */
   int print_motd;		/* If true, print /etc/motd. */
+  int x11_forwarding;		/* If true, permit inet (spoofing) X11 fwd. */
+  int strict_modes;		/* If true, require string home dir modes. */
+  int keepalives;		/* If true, set SO_KEEPALIVE. */
   SyslogFacility log_facility;	/* Facility for system logging. */
   int rhosts_authentication;	/* If true, permit rhosts authentication. */
   int rhosts_rsa_authentication;/* If true, permit rhosts RSA authentication.*/
   int rsa_authentication;	/* If true, permit RSA authentication. */
   int password_authentication;  /* If true, permit password authentication. */
+  int permit_empty_passwd;      /* If false, do not permit empty passwords. */
   unsigned int num_allow_hosts;
   char *allow_hosts[MAX_ALLOW_HOSTS];
   unsigned int num_deny_hosts;
