@@ -14,8 +14,14 @@ Adds an identity to the authentication server, or removes an identity.
 */
 
 /*
- * $Id: ssh-add.c,v 1.3 1995/08/29 22:24:21 ylo Exp $
+ * $Id: ssh-add.c,v 1.4 1995/10/02 01:27:34 ylo Exp $
  * $Log: ssh-add.c,v $
+ * Revision 1.4  1995/10/02  01:27:34  ylo
+ * 	Loop asking for a proper passphrase until the user aborts or
+ * 	gives an empty passphrase.  (This avoids problems of
+ * 	accidentally typing the wrong passphrase without noticing it
+ * 	when using ssh-add from .Xsession.real.)
+ *
  * Revision 1.3  1995/08/29  22:24:21  ylo
  * 	Added delete_all.
  *
@@ -92,27 +98,34 @@ void add_file(const char *filename)
   RSAPrivateKey key;
   RSAPublicKey public_key;
   AuthenticationConnection *ac;
-  char *saved_comment, *comment;
+  char *saved_comment, *comment, *pass;
+  int first;
   
   if (!load_public_key(filename, &public_key, &saved_comment))
     {
       printf("Bad key file %s: %s\n", filename, strerror(errno));
       return;
     }
+  rsa_clear_public_key(&public_key);
   
-  if (!load_private_key(filename, "", &key, &comment))
+  pass = xstrdup("");
+  first = 1;
+  while (!load_private_key(filename, pass, &key, &comment))
     {
       char buf[1024];
-      char *pass;
       FILE *f;
       
-      printf("Need passphrase for %s (%s).\n", filename, saved_comment);
+      /* Free the old passphrase. */
+      memset(pass, 0, strlen(pass));
+      xfree(pass);
+
+      /* Ask for a passphrase. */
       if (getenv("DISPLAY") && !isatty(fileno(stdin)))
 	{
-	  printf("Executing ssh-askpass to query the password...\n");
-	  fflush(stdout);
-	  fflush(stderr);
-	  f = popen("ssh-askpass", "r");
+	  sprintf(buf, "ssh-askpass '%sEnter passphrase for %.100s'", 
+		  first ? "" : "You entered wrong passphrase.  ", 
+		  saved_comment);
+	  f = popen(buf, "r");
 	  if (!fgets(buf, sizeof(buf), f))
 	    {
 	      pclose(f);
@@ -126,20 +139,22 @@ void add_file(const char *filename)
 	}
       else
 	{
-	  sprintf(buf, "Enter passphrase: ");
-	  pass = read_passphrase(buf, 1);
+	  if (first)
+	    printf("Need passphrase for %s (%s).\n", filename, saved_comment);
+	  else
+	    printf("Bad passphrase.\n");
+	  pass = read_passphrase("Enter passphrase: ", 1);
+	  if (strcmp(pass, "") == 0)
+	    {
+	      xfree(saved_comment);
+	      xfree(pass);
+	      return;
+	    }
 	}
-	  
-      if (!load_private_key(filename, pass, &key, &comment))
-	{
-	  memset(pass, 0, strlen(pass));
-	  fprintf(stderr, 
-		  "Bad passphrase, file not readable, or invalid keyfile.\n");
-	  xfree(saved_comment);
-	  exit(1);
-	}
-      memset(pass, 0, strlen(pass));
+      first = 0;
     }
+  memset(pass, 0, strlen(pass));
+  xfree(pass);
 
   xfree(saved_comment);
 
@@ -247,13 +262,3 @@ int main(int ac, char **av)
     }
   exit(0);
 }
-
-void fatal(const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  vfprintf(stderr, fmt, args);
-  fprintf(stderr, "\n");
-  va_end(args);
-  exit(1);
-}
