@@ -14,8 +14,33 @@ Generic header file for ssh.
 */
 
 /*
- * $Id: ssh.h,v 1.9 1995/08/31 09:23:42 ylo Exp $
+ * $Id: ssh.h,v 1.16 1995/10/02 01:31:13 ylo Exp $
  * $Log: ssh.h,v $
+ * Revision 1.16  1995/10/02  01:31:13  ylo
+ * 	Moved sshd.pid to PIDDIR.  Also changed file name.
+ * 	Added SSH_HOSTS_EQUIV (shosts.equiv).
+ *
+ * Revision 1.15  1995/09/27  02:16:24  ylo
+ * 	Added SSH_USER_RC and SSH_SYSTEM_RC.
+ *
+ * Revision 1.14  1995/09/25  00:02:06  ylo
+ * 	Added client_loop.
+ * 	Added screen number arguments.
+ * 	Added connection_attempts.
+ *
+ * Revision 1.13  1995/09/22  22:23:41  ylo
+ * 	Changed argument list of ssh_login.
+ *
+ * Revision 1.12  1995/09/21  17:14:08  ylo
+ * 	Added original_real_uid argument to ssh_connect.
+ *
+ * Revision 1.11  1995/09/10  22:47:59  ylo
+ * 	Added server_loop.
+ * 	Added original_real_uid parameter to ssh_login.
+ *
+ * Revision 1.10  1995/09/09  21:26:46  ylo
+ * /m/shadows/u2/users/ylo/ssh/README
+ *
  * Revision 1.9  1995/08/31  09:23:42  ylo
  * 	Added support for ETCDIR.
  *
@@ -54,13 +79,35 @@ Generic header file for ssh.
 #include "randoms.h"
 #include "cipher.h"
 
+/* The default cipher used if IDEA is not supported by the remote host. 
+   It is recommended that this be one of the mandatory ciphers (DES, 3DES),
+   though that is not required. */
+#define SSH_FALLBACK_CIPHER	SSH_CIPHER_3DES
+
+/* Cipher used for encrypting authentication files. */
+#ifdef WITHOUT_IDEA
+#define SSH_AUTHFILE_CIPHER	SSH_CIPHER_3DES
+#else
+#define SSH_AUTHFILE_CIPHER	SSH_CIPHER_IDEA
+#endif
+
+/* Default port number. */
+#define SSH_DEFAULT_PORT	22
+
+/* Maximum number of TCP/IP ports forwarded per direction. */
+#define SSH_MAX_FORWARDS_PER_DIRECTION	100
+
+/* Maximum number of RSA authentication identity files that can be specified
+   in configuration files or on the command line. */
+#define SSH_MAX_IDENTITY_FILES		100
+
 /* Major protocol version.  Different version indicates major incompatiblity
    that prevents communication.  */
 #define PROTOCOL_MAJOR		1
 
 /* Minor protocol version.  Different version indicates minor incompatibility
    that does not prevent interoperation. */
-#define PROTOCOL_MINOR		1
+#define PROTOCOL_MINOR		3
 
 /* Name for the service.  The port named by this service overrides the default
    port if present. */
@@ -82,7 +129,7 @@ only by root, whereas ssh_config should be world-readable. */
 
 /* The process id of the daemon listening for connections is saved
    here to make it easier to kill the correct daemon when necessary. */
-#define SSH_DAEMON_PID_FILE	ETCDIR "/sshd_pid"
+#define SSH_DAEMON_PID_FILE	PIDDIR "/sshd.pid"
 
 /* The directory in user\'s home directory in which the files reside.
    The directory should be world-readable (though not all files are). */
@@ -117,6 +164,16 @@ only by root, whereas ssh_config should be world-readable. */
    root.) */
 #define SSH_USER_PERMITTED_KEYS	".ssh/authorized_keys"
 
+/* Per-user and system-wide ssh "rc" files.  These files are executed with
+   /bin/sh before starting the shell or command if they exist.  They
+   will be passed "proto cookie" as arguments if X11 forwarding with
+   spoofing is in use.  xauth will be run if neither of these exists. */
+#define SSH_USER_RC		".ssh/rc"
+#define SSH_SYSTEM_RC		ETCDIR "/sshrc"
+
+/* Ssh-only version of /etc/hosts.equiv. */
+#define SSH_HOSTS_EQUIV		ETCDIR "/shosts.equiv"
+
 /* Additionally, the daemon may use ~/.rhosts and /etc/hosts.equiv if 
    rhosts authentication is enabled. */
 
@@ -141,16 +198,6 @@ only by root, whereas ssh_config should be world-readable. */
    authentication socket. */
 #define SSH_AUTHSOCKET_ENV_NAME	"SSH_AUTHENTICATION_SOCKET"
 
-/* Cipher used for encrypting authentication files. */
-#ifdef WITHOUT_IDEA
-#define SSH_AUTHFILE_CIPHER	SSH_CIPHER_3DES
-#else
-#define SSH_AUTHFILE_CIPHER	SSH_CIPHER_IDEA
-#endif
-
-/* Default port number. */
-#define SSH_DEFAULT_PORT	22
-
 /* Force host key length and server key length to differ by at least this
    many bits.  This is to make double encryption with rsaref work. */
 #define SSH_KEY_BITS_RESERVED		128
@@ -159,13 +206,6 @@ only by root, whereas ssh_config should be world-readable. */
    protocol.)  */
 #define SSH_SESSION_KEY_LENGTH		32
 
-/* Maximum number of TCP/IP ports forwarded per direction. */
-#define SSH_MAX_FORWARDS_PER_DIRECTION	100
-
-/* Maximum number of RSA authentication identity files that can be specified
-   in configuration files or on the command line. */
-#define SSH_MAX_IDENTITY_FILES		100
-
 /* Authentication methods.  New types can be added, but old types should not
    be removed for compatibility.  The maximum allowed value is 31. */
 #define SSH_AUTH_RHOSTS		1
@@ -173,6 +213,10 @@ only by root, whereas ssh_config should be world-readable. */
 #define SSH_AUTH_PASSWORD	3
 #define SSH_AUTH_RHOSTS_RSA	4
 
+/* Protocol flags.  These are bit masks. */
+#define SSH_PROTOFLAG_SCREEN_NUMBER	1 /* X11 forwarding includes screen */
+#define SSH_PROTOFLAG_HOST_IN_FWD_OPEN	2 /* forwarding opens contain host */
+
 /* Definition of message types.  New values can be added, but old values
    should not be removed or without careful consideration of the consequences
    for compatibility.  The maximum value is 254; value 255 is reserved
@@ -204,7 +248,7 @@ only by root, whereas ssh_config should be world-readable. */
 #define SSH_MSG_CHANNEL_DATA			23	/* ch,data (int,str) */
 #define SSH_MSG_CHANNEL_CLOSE			24	/* channel (int) */
 #define SSH_MSG_CHANNEL_CLOSE_CONFIRMATION	25	/* channel (int) */
-#define SSH_CMSG_X11_REQUEST_FORWARDING		26	/* */
+/*      SSH_CMSG_X11_REQUEST_FORWARDING         26         OBSOLETE */
 #define SSH_SMSG_X11_OPEN			27	/* channel (int) */
 #define SSH_CMSG_PORT_FORWARD_REQUEST		28	/* p,host,hp (i,s,i) */
 #define SSH_MSG_PORT_OPEN			29	/* ch,h,p (i,s,i) */
@@ -212,9 +256,14 @@ only by root, whereas ssh_config should be world-readable. */
 #define SSH_SMSG_AGENT_OPEN			31	/* port (int) */
 #define SSH_MSG_IGNORE				32	/* string */
 #define SSH_CMSG_EXIT_CONFIRMATION		33	/* */
-#define SSH_CMSG_X11_FWD_WITH_AUTH_SPOOFING	34	/* proto,data (s,s) */
+#define SSH_CMSG_X11_REQUEST_FORWARDING		34	/* proto,data (s,s) */
 #define SSH_CMSG_AUTH_RHOSTS_RSA		35	/* user,mod (s,mpi) */
 #define SSH_MSG_DEBUG				36	/* string */
+#define SSH_CMSG_REQUEST_COMPRESSION		37	/* level 1-9 (int) */
+
+/* Includes that need definitions above. */
+
+#include "readconf.h"
 
 /*------------ definitions for login.c -------------*/
 
@@ -227,7 +276,7 @@ unsigned long get_last_login_time(uid_t uid, const char *logname,
 /* Records that the user has logged in.  This does many things normally
    done by login(1). */
 void record_login(int pid, const char *ttyname, const char *user, uid_t uid,
-		  const char *host);
+		  const char *host, struct sockaddr_in *addr);
 
 /* Records that the user has logged out.  This does many thigs normally
    done by login(1) or init. */
@@ -238,38 +287,40 @@ void record_logout(int pid, const char *ttyname);
 /* Opens a TCP/IP connection to the remote server on the given host.  If
    port is 0, the default port will be used.  If anonymous is zero,
    a privileged port will be allocated to make the connection. 
-   This requires super-user privileges if anonymous is false. */
-int ssh_connect(const char *host, int port, int anonymous);
+   This requires super-user privileges if anonymous is false. 
+   Connection_attempts specifies the maximum number of tries, one per
+   second.  This returns true on success, and zero on failure.  If the
+   connection is successful, this calls packet_set_connection for the
+   connection. */
+int ssh_connect(const char *host, int port, int connection_attempts,
+		int anonymous, uid_t original_real_uid,
+		const char *proxy_command, RandomState *random_state);
 
 /* Starts a dialog with the server, and authenticates the current user on the
    server.  This does not need any extra privileges.  The basic connection
    to the server must already have been established before this is called. 
-   User is the remote user; if it is NULL, the current local user name will
-   be used.  Anonymous indicates that no rhosts authentication will be used.
    If login fails, this function prints an error and never returns. 
-   This function does not require super-user privileges. 
    This initializes the random state, and leaves it initialized (it will also
    have references from the packet module). */
 void ssh_login(RandomState *state, int host_key_valid, RSAPrivateKey *host_key,
-	       int sock, const char *host, const char *user, 
-	       unsigned int num_identity_files, char **identity_files,
-	       int rhosts_authentication, int rhosts_rsa_authentication,
-	       int rsa_authentication,
-	       int password_authentication, int cipher_type,
-	       const char *system_hostfile, const char *user_hostfile);
+	       const char *host, Options *options, uid_t original_real_uid);
 
 /*------------ Definitions for various authentication methods. -------*/
 
 /* Tries to authenticate the user using the .rhosts file.  Returns true if
-   authentication succeeds.  */
-int auth_rhosts(struct passwd *pw, const char *client_user);
+   authentication succeeds.  If ignore_rhosts is non-zero, this will not
+   consider .rhosts and .shosts (/etc/hosts.equiv will still be used). 
+   If strict_modes is true, checks ownership and modes of .rhosts/.shosts. */
+int auth_rhosts(struct passwd *pw, const char *client_user,
+		int ignore_rhosts, int strict_modes);
 
 /* Tries to authenticate the user using the .rhosts file and the host using
    its host key.  Returns true if authentication succeeds. */
 int auth_rhosts_rsa(RandomState *state,
 		    struct passwd *pw, const char *client_user,
 		    unsigned int bits, MP_INT *client_host_key_e,
-		    MP_INT *client_host_key_n);
+		    MP_INT *client_host_key_n, int ignore_rhosts,
+		    int strict_modes);
 
 /* Tries to authenticate the user using password.  Returns true if
    authentication succeeds. */
@@ -284,14 +335,24 @@ int auth_rsa(struct passwd *pw, MP_INT *client_n, RandomState *state);
    over the key.  Skips any whitespace at the beginning and at end. */
 int auth_rsa_read_key(char **cpp, unsigned int *bitsp, MP_INT *e, MP_INT *n);
 
+/* Returns the name of the machine at the other end of the socket.  The
+   returned string should be freed by the caller. */
+char *get_remote_hostname(int socket);
+
 /* Return the canonical name of the host in the other side of the current
    connection (as returned by packet_get_connection).  The host name is
    cached, so it is efficient to call this several times. */
-const char *get_canonical_hostname();
+const char *get_canonical_hostname(void);
 
 /* Returns the remote IP address as an ascii string.  The value need not be
    freed by the caller. */
-const char *get_remote_ipaddr();
+const char *get_remote_ipaddr(void);
+
+/* Returns the port number of the peer of the socket. */
+int get_peer_port(int sock);
+
+/* Returns the port number of the remote host. */
+int get_remote_port(void);
 
 /* Tries to match the host name (which must be in all lowercase) against the
    comma-separated sequence of subpatterns (each possibly preceded by ! to 
@@ -392,11 +453,24 @@ void error(const char *fmt, ...);
    This call never returns. */
 void fatal(const char *fmt, ...);
 
+/* Registers a cleanup function to be called by fatal() before exiting. 
+   It is permissible to call fatal_remove_cleanup for the function itself
+   from the function. */
+void fatal_add_cleanup(void (*proc)(void *context), void *context);
+
+/* Removes a cleanup frunction to be called at fatal(). */
+void fatal_remove_cleanup(void (*proc)(void *context), void *context);
+
 /*---------------- definitions for x11.c ------------------*/
 
 
-/* Allocate a new channel object and set its type and socket. */
-int channel_allocate(int type, int sock);
+/* Sets specific protocol options. */
+void channel_set_options(int hostname_in_open);
+
+/* Allocate a new channel object and set its type and socket.  Remote_name
+   must have been allocated with xmalloc; this will free it when the channel
+   is freed. */
+int channel_allocate(int type, int sock, char *remote_name);
 
 /* Free the channel and close its socket. */
 void channel_free(int channel);
@@ -417,7 +491,7 @@ void channel_output_poll(void);
 void channel_input_data(void);
 
 /* Returns true if no channel has too much buffered data. */
-int channel_not_very_much_buffered_data();
+int channel_not_very_much_buffered_data(void);
 
 /* This is called after receiving CHANNEL_CLOSE. */
 void channel_input_close(void);
@@ -445,6 +519,11 @@ int channel_max_fd(void);
 /* Returns true if there is still an open channel over the connection. */
 int channel_still_open(void);
 
+/* Returns a string containing a list of all open channels.  The list is
+   suitable for displaying to the user.  It uses crlf instead of newlines.
+   The caller should free the string with xfree. */
+char *channel_open_message(void);
+
 /* Initiate forwarding of connections to local port "port" through the secure
    channel to host:port from remote side.  This never returns if there
    was an error. */
@@ -476,12 +555,12 @@ void channel_input_port_open(void);
 
 /* Creates a port for X11 connections, and starts listening for it.
    Returns the display name, or NULL if an error was encountered. */
-char *x11_create_display(void);
+char *x11_create_display(int screen);
 
 /* Creates an internet domain socket for listening for X11 connections. 
    Returns a suitable value for the DISPLAY variable, or NULL if an error
    occurs. */
-char *x11_create_display_inet(void);
+char *x11_create_display_inet(int screen);
 
 /* This is called when SSH_SMSG_X11_OPEN is received.  The packet contains
    the remote channel number.  We should do whatever we want, and respond
@@ -529,4 +608,20 @@ char *tilde_expand_filename(const char *filename, uid_t my_uid);
    variable or the user id. */
 int get_permanent_fd(const char *pathname);
 
+/* Performs the interactive session.  This handles data transmission between
+   the client and the program.  Note that the notion of stdin, stdout, and
+   stderr in this function is sort of reversed: this function writes to
+   stdin (of the child program), and reads from stdout and stderr (of the
+   child program). */
+void server_loop(int pid, int fdin, int fdout, int fderr);
+
+/* Client side main loop for the interactive session. */
+int client_loop(int have_pty, int escape_char);
+
+/* Linked list of custom environment strings (see auth-rsa.c). */
+struct envstring {
+  struct envstring *next;
+  char *s;
+};
+
 #endif /* SSH_H */
